%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                Bar and Hinge for Curved-Crease Origami                %%
%         Modification by: Steven R. Woodruff (stevenrw@umich.edu)        %
%             Base code written by: Ke Liu (ke.liu@gatech.edu)            %
% Ref: S. R. Woodruff, E. T. Filipov (2020). 'A bar and hinge formulation %
%          for structural analysis of curved-crease origami.' Submitted.  %
%      K. Liu, G. H. Paulino (2017). 'Nonlinear mechanics of non-rigid    %
%          origami - An efficient computational approach.' PRSA.          %
%      K. Liu, G. H. Paulino (2016). 'MERLIN: A MATLAB implementation to  % 
%          capture highly nonlinear behavior of non-rigid origami.'       %
%          Proceedings of IASS Annual Symposium 2016.                     %
%      E. T. Filipov, K. Liu, T. Tachi, M. Schenk, G. H. Paulino (2017).  %
%          'Bar and hinge models for scalable analysis of origami.'  IJSS %
%      K. Liu, G. H. Paulino (2018). 'Highly efficient nonlinear          %
%          structural analysis of origami assemblages using the MERLIN2   %
%          software.' Origami^7.                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Reset workspace
clearvars; close all; clc;
% Add required paths to analysis functions and fold pattern functions
addpath('Functions'); addpath('Geometries')

%% Input Parameters

% Folding examples
    testType = 'FullAnnulus';
%     testType = 'SineWave';
%     testType = 'CurvedSquare';
%     testType = 'Canopy';
%     testType = 'CutAnnulus';
%     testType = 'FoldedStrip';

% Post-fold loading examples
%     testType = 'LoadedAnnulus';
    

    InputData = struct('restAngle', 90, ...             % Prescribed folding rest dihedral angle (\phi_R) [deg]
                       'elasticModulus', 4000, ...       % Elastic modulus of the sheet (E)
                       'thickness', 0.127, ...            % Thickness of the sheet (t)
                       'lengthScale', 10,...           % folding length scale factor (L*)
                       'loadMagnitude', 0.1, ...        % Load magnitude (displacement or force)
                       'loadType', 'Displacement', ...  % Either 'Force' or 'Displacement' controlled test
                       'numberIncrements', 20, ...      % Specifies the number of displacement or force steps (increments)
                       'maxIterations', 50, ...         % Specifies the maximum number of step iterations
                       'testType', testType, ...        % Specifies specific test (see example options above)
                       'numberDivisions', 40, ...      % This controls the discretization size
                       'plotNodes', 'yes', ...          % Plots the flat nodes, bars, and boundary conditions
                       'plotEnergy', 'yes', ...         % Plots the strain energy in the elements
                       'plotDeformedShape', 'yes', ...  % Plots the deformed shape after analysis
                       'plotReferenceShape', 'yes',...  % Plots the flat, referece configuration under deformed shape
                       'plotIncrement','end');          % Value determines which step is plotted (or end)
                  
    clear testType  
    
%% Test Inputs
    [InputData, inputTest] = Test_Input(InputData);

    if inputTest == 0
        fprintf('Input variable error(s). See above.\n\n')
        return
    elseif inputTest == 2
        fprintf('Possible error. See above. Hit enter to proceed.\n\n')
        pause
    end; clear inputTest
    
%% Generate mesh
         
    if strcmp(InputData.testType,'FullAnnulus')
        InputData.numberDivisions = 4*round(InputData.numberDivisions/4); % Annulus BCs require divisions as a multiple of 4
        InputData = Geometry_FullAnnulus(InputData);
        InputData.foldStep = true;
    elseif strcmp(InputData.testType,'SineWave')
        InputData = Geometry_SineWave(InputData);
        InputData.foldStep = true;
    elseif strcmp(InputData.testType,'CurvedSquare')
        InputData = Geometry_CurvedSquare(InputData);
        InputData.foldStep = true;
    elseif strcmp(InputData.testType,'Canopy')
        [InputData] = Geometry_Canopy(InputData);
        InputData.foldStep = true;
    elseif strcmp(InputData.testType,'CutAnnulus')
        [InputData] = Geometry_CutAnnulus(InputData);
        InputData.foldStep = true;
    elseif strcmp(InputData.testType,'FoldedStrip')
        [InputData] = Geometry_FoldedStrip(InputData);
        InputData.foldStep = false;
    elseif strcmp(InputData.testType,'LoadedAnnulus')
        [InputData] = Geometry_LoadedAnnulus(InputData);
        InputData.foldStep = false;
    end

%% Perform analysis
    % Assemble input data
    [PreprocessData] = Prepare_Data(InputData);
       
%% Run Analysis
    % Perform path-following analysis
    [PostprocessData] = Path_Analysis(InputData, PreprocessData);
    % Postprocess output data
    PostprocessData = Post_Process(InputData, PreprocessData, PostprocessData); 

%% Plot results
    Plot_Results(InputData, PreprocessData, PostprocessData);
    